; Copyright (c) 1993, 1994 Richard Kelsey and Jonathan Rees.  See file COPYING.


; Using sockets from Scheme 48.

; Server side routines:
;   (open-socket port-number-or-zero) => socket
;   (socket-port-number socket) => port number
;     This is essential if the argument to open-socket was zero.
;   (socket-listen s) => two values: input port and output port
;     Blocks until an outside party connects to the socket's port number.
;   (close-socket s)
;     Do this after you've done all the socket-listens you want to.

; Client side routine:
;   (socket-client host-name port-number) =>
;       two values: input port and output port


; --------------------
; Client.

(define (socket-client host-name port-number)
  (fd->ports (socket-connect host-name port-number)))

; Old server interface, for compatibility.

(define (socket-server)
  (let ((socket (open-socket 0)))
    (values (socket-port-number socket)
	    (lambda ()
	      (socket-listen socket)))))

; --------------------
; Server calls

(define-record-type socket :socket
  (make-socket ch pn)
  socket?
  (ch socket-channel)
  (pn socket-port-number))

(define-record-discloser :socket
  (lambda (s) `(socket ,(socket-port-number s))))

(define (open-socket port-number)
  (let* ((socket-fd (internet-stream-socket))
	 (channel (open-channel socket-fd (enum open-channel-option
						special-input-channel))))
    (make-socket channel (socket-bind socket-fd port-number))))

(define (close-socket socket)
  (close-channel (socket-channel socket)))

(define (socket-listen socket)
  (fd->ports (socket-accept (socket-channel socket))))

; --------------------
; vm-extension calls

(define (internet-stream-socket)
  (vm-extension 20 #f))

(define (socket-bind s port-number)
  (vm-extension 21 (cons s port-number)))

(define (socket-accept socket-channel)
  (let ((fd (vm-extension 22 socket-channel)))
    (cond ((= fd -1)  ; not ready
	   (block-on-channel socket-channel)
	   (socket-accept socket-channel))
	  (else
	   fd))))

(define (socket-connect host-name port-number)
  (vm-extension 23 (cons host-name port-number)))

; We duplicate the fd so that the input and output channels can be
; closed independently.

(define (fd->ports fd)
  (let ((out-fd (duplicate-fd fd)))
    (values (input-channel->port (open-channel fd (enum open-channel-option
							raw-input-channel))
				 1024)  ; buffer size
	    (output-channel->port (open-channel out-fd
						(enum open-channel-option
						      raw-output-channel))
				  1024))))

(define (duplicate-fd fd)
  (vm-extension 29 fd))
